<?php
/**
 * License: GPLv3
 * License URI: https://www.gnu.org/licenses/gpl.txt
 * Copyright 2014-2021 Jean-Sebastien Morisset (https://wpsso.com/)
 */

if ( ! defined( 'ABSPATH' ) ) {

	die( 'These aren\'t the droids you\'re looking for.' );
}

if ( ! class_exists( 'WpssoPlmFiltersOptions' ) ) {

	class WpssoPlmFiltersOptions {

		private $p;	// Wpsso class object.
		private $a;	// WpssoPlm class object.

		/**
		 * Instantiated by WpssoPlmFilters->__construct().
		 */
		public function __construct( &$plugin, &$addon ) {

			$this->p =& $plugin;
			$this->a =& $addon;

			$this->p->util->add_plugin_filters( $this, array( 
				'option_type'          => 2,
				'save_setting_options' => 3,
				'save_md_options'      => 2,
				'get_md_defaults'      => 1,
				'get_md_options'       => 2,
				'get_place_options'    => 3,
			) );
		}

		/**
		 * Return the sanitation type for a given option key.
		 */
		public function filter_option_type( $type, $base_key ) {

			if ( ! empty( $type ) ) {	// Return early if we already have a type.

				return $type;

			} elseif ( 0 !== strpos( $base_key, 'plm_' ) ) {	// Nothing to do.

				return $type;
			}

			switch ( $base_key ) {

				case 'plm_place_edit_id':	// Edit a Place.

					return 'numeric';

				case ( preg_match( '/^plm_place_(country|schema_type)$/', $base_key ) ? true : false ):

					return 'not_blank';

				case ( preg_match( '/^plm_place_(name|name_alt|desc|phone|street_address|city|state|postal_code|zipcode)$/', $base_key ) ? true : false ):
				case ( preg_match( '/^plm_place_(phone|price_range|cuisine)$/', $base_key ) ? true : false ):

					return 'ok_blank';

				case ( preg_match( '/^plm_place_(currencies_accepted|payment_accepted)$/', $base_key ) ? true : false ):

					return 'csv_blank';

				case ( preg_match( '/^plm_place_(latitude|longitude|altitude|service_radius|po_box_number)$/', $base_key ) ? true : false ):

					return 'blank_num';

				case ( preg_match( '/^plm_place_(day_[a-z]+|midday)_(open|close)$/', $base_key ) ? true : false ):

					return 'time';	// Empty or 'none' string, or time as hh:mm or hh:mm:ss.

				case ( preg_match( '/^plm_place_season_(from|to)_date$/', $base_key ) ? true : false ):

					return 'date';	// Empty or 'none' string, or date as yyyy-mm-dd.

				case 'plm_place_menu_url':

					return 'url';

				case 'plm_place_order_urls':

					return 'csv_urls';

				case 'plm_place_accept_res':

					return 'checkbox';
			}

			return $type;
		}

		/**
		 * $network is true if saving multisite network settings.
		 */
		public function filter_save_setting_options( array $opts, $network, $upgrading ) {

			if ( $network ) {

				return $opts;	// Nothing to do.
			}

			/**
			 * Example:
			 *
			 * $place_names = Array (
			 *	[0] => First Place
			 *	[1] => Second Place
			 * )
			 */
			$place_names    = SucomUtil::get_multi_key_locale( 'plm_place_name', $opts, $add_none = false );
			$place_last_num = SucomUtil::get_last_num( $place_names );
			$def_place_type = WpssoPlmConfig::$cf[ 'opt' ][ 'def_plm_place' ][ 'plm_place_schema_type' ];

			foreach ( $place_names as $place_id => $place_name ) {

				$place_name      = trim( $place_name );	// Just in case.
				$place_id_locale = SucomUtil::get_key_locale( $place_id, $opts );	// Use to get localized values.

				/**
				 * Remove empty "New Place".
				 */
				if ( ! empty( $opts[ 'plm_place_delete_' . $place_id ] ) || ( '' === $place_name && $place_last_num === $place_id ) ) {

					/**
					 * Maybe reset the currently selected place ID.
					 */
					if ( isset( $opts[ 'plm_place_edit_id' ] ) && $opts[ 'plm_place_edit_id' ] === $place_id ) {

						unset( $opts[ 'plm_place_edit_id' ] );
					}

					/**
					 * Remove the place, including all localized keys.
					 */
					$opts = SucomUtil::preg_grep_keys( '/^plm_place_.*_' . $place_id . '(#.*)?$/', $opts, $invert = true );

					continue;	// Check the next place.
				}

				/**
				 * Just in case, make sure the default locale plm_place_name option has a value.
				 */
				if ( empty( $opts[ 'plm_place_name_' . $place_id ] ) ) {	// Check the default locale.

					if ( empty( $place_name ) ) {

						$place_name = sprintf( _x( 'Place #%d', 'option value', 'wpsso-plm' ), $place_id );
					}

					$opts[ 'plm_place_name_' . $place_id ] = $place_name;
				}

				/**
				 * The plm_place_img options are localized.
				 */
				if ( empty( $opts[ 'plm_place_img_id_' . $place_id_locale ] ) ) {	// Image id 0 is not valid.

					unset(
						$opts[ 'plm_place_img_id_' . $place_id_locale ],
						$opts[ 'plm_place_img_id_lib_' . $place_id_locale ]
					);

				} else {

					/**
					 * Remove the image URL options if we have an image ID.
					 */
					unset(
						$opts[ 'plm_place_img_url_' . $place_id_locale ],
						$opts[ 'plm_place_img_url:width_' . $place_id_locale ],
						$opts[ 'plm_place_img_url:height_' . $place_id_locale ]
					);

					/**
					 * Get the location image and issue an error if the original image is too small.
					 *
					 * Only check on a manual save, not an options upgrade action (ie. when a new add-on is activated).
					 */
					if ( ! $upgrading ) {

						$this->check_location_image_sizes( $opts, $place_id );
					}
				} 
			}

			$opts = SucomUtil::preg_grep_keys( $key_pattern = '/^plm_place_delete_/', $opts, $invert = true );	// Just in case.

			return $opts;
		}

		public function filter_save_md_options( $md_opts, $mod ) {

			$md_opts = $this->update_md_opts( $md_opts, $mod );

			return $md_opts;
		}

		public function filter_get_md_defaults( $md_defs ) {

			$md_defs = array_merge( $md_defs, WpssoPlmConfig::$cf[ 'opt' ][ 'def_plm_place' ], array(
				'plm_place_country'             => $this->p->options[ 'og_def_country' ],
				'plm_place_timezone'            => $this->p->options[ 'og_def_timezone' ],
				'plm_place_currencies_accepted' => $this->p->options[ 'og_def_currency' ],
			) );

			return $md_defs;
		}

		public function filter_get_md_options( $md_opts, $mod ) {

			$md_opts = $this->update_md_opts( $md_opts, $mod );

			return $md_opts;
		}

		/**
		 * Called by WpssoPlmFiltersOptions->filter_save_md_options() to save the option values, and
		 * WpssoPlmFiltersOptions->filter_get_md_options() to define and disable some option values.
		 */
		private function update_md_opts( $md_opts, $mod ) {

			$place_id       = isset( $md_opts[ 'schema_place_id' ] ) ? $md_opts[ 'schema_place_id' ] : 'none';	// Just in case.
			$place_type     = false;
			$def_place_type = WpssoPlmConfig::$cf[ 'opt' ][ 'def_plm_place' ][ 'plm_place_schema_type' ];

			/**
			 * Get the Schema type of this place ID.
			 */
			if ( '' === $place_id || 'none' === $place_id ) {	// Nothing to do.

				$md_opts = SucomUtil::preg_grep_keys( '/^plm_place_/', $md_opts, $invert = true );

				return $md_opts;	// Stop here.

			} elseif ( 'custom' === $place_id ) {	// Value is "custom".

				$place_type = empty( $md_opts[ 'plm_place_schema_type' ] ) ? $def_place_type : $md_opts[ 'plm_place_schema_type' ];

			} elseif ( is_numeric( $place_id ) ) {	// Value is "0" or more.

				$place_opts = WpssoPlmPlace::get_id( $place_id, $mod );
				$place_type = empty( $place_opts[ 'plm_place_schema_type' ] ) ? $def_place_type : $place_opts[ 'plm_place_schema_type' ];

			} else {	// Just in case.

				return $md_opts;	// Stop here.
			}

			$md_opts[ 'og_type' ]    = 'place';	// Overwrite the WPSSO option value.
			$md_opts[ 'og_type:is' ] = 'disabled';

			if ( $place_type ) {	// Just in case.

				$md_opts[ 'schema_type' ]    = $place_type;
				$md_opts[ 'schema_type:is' ] = 'disabled';

				$md_opts[ 'schema_organization_id' ]    = 'none';
				$md_opts[ 'schema_organization_id:is' ] = 'disabled';
			}

			return $md_opts;
		}

		public function filter_get_place_options( $place_opts, $mod, $place_id ) {

			if ( false !== $place_opts ) {	// First come, first served.

				if ( $this->p->debug->enabled ) {

					$this->p->debug->log( 'exiting early: place_opts array already defined' );
				}

				return $place_opts;
			}

			/**
			 * $place_id can be 'none', 'custom', or a place ID number.
			 */
			if ( 'custom' === $place_id || is_numeric( $place_id ) ) {	// Allow for place ID 0.

				$plm_place_opts = WpssoPlmPlace::get_id( $place_id, $mod );

				if ( is_array( $plm_place_opts ) ) {	// Just in case.

					return SucomUtil::preg_grep_keys( '/^plm_place_/', $plm_place_opts, $invert = false, $replace = 'place_' );
				}
			}

			return $place_opts;
		}

		/**
		 * Get the location image and issue an error if the original image is too small.
		 */
		private function check_location_image_sizes( $opts, $opt_num = null ) {

			$name_transl        = SucomUtil::get_key_value( 'plm_place_name_' . $opt_num, $opts, 'current' );
			$context_transl     = sprintf( __( 'saving place "%1$s"', 'wpsso-plm' ), $name_transl );
			$settings_page_link = $this->p->util->get_admin_url( 'plm-general' );

			$this->p->util->maybe_set_ref( $settings_page_link, $mod = false, $context_transl );

			/**
			 * $size_names can be a keyword (ie. 'opengraph' or 'schema'), a registered size name, or an array of size names.
			 */
			$mt_images = $this->p->media->get_mt_opts_images( $opts, $size_names = 'schema', $img_pre = 'plm_place_img', $opt_num );

			$this->p->util->maybe_unset_ref( $settings_page_link );
		}
	}
}
