<?php
/**
 * License: GPLv3
 * License URI: https://www.gnu.org/licenses/gpl.txt
 * Copyright 2014-2021 Jean-Sebastien Morisset (https://wpsso.com/)
 */

if ( ! defined( 'ABSPATH' ) ) {

	die( 'These aren\'t the droids you\'re looking for.' );
}

if ( ! class_exists( 'WpssoPlmFilters' ) ) {

	class WpssoPlmFilters {

		private $p;	// Wpsso class object.
		private $a;	// WpssoPlm class object.
		private $edit;	// WpssoPlmFiltersEdit class object.
		private $msgs;	// WpssoPlmFiltersMessages class object.
		private $opts;	// WpssoPlmFiltersOptions class object.
		private $upg;	// WpssoPlmFiltersUpgrade class object.

		/**
		 * Instantiated by WpssoPlm->init_objects().
		 */
		public function __construct( &$plugin, &$addon ) {

			static $do_once = null;

			if ( true === $do_once ) {

				return;	// Stop here.
			}

			$do_once = true;

			$this->p =& $plugin;
			$this->a =& $addon;

			require_once WPSSOPLM_PLUGINDIR . 'lib/filters-options.php';

			$this->opts = new WpssoPlmFiltersOptions( $plugin, $addon );

			require_once WPSSOPLM_PLUGINDIR . 'lib/filters-upgrade.php';

			$this->upg = new WpssoPlmFiltersUpgrade( $plugin, $addon );

			$this->p->util->add_plugin_filters( $this, array( 
				'og_type'                                    => 3,
				'og_seed'                                    => 2,
				'json_array_schema_type_ids'                 => 2,
				'json_prop_https_schema_org_potentialaction' => 5,
			) );

			if ( is_admin() ) {

				require_once WPSSOPLM_PLUGINDIR . 'lib/filters-edit.php';

				$this->edit = new WpssoPlmFiltersEdit( $plugin, $addon );

				require_once WPSSOPLM_PLUGINDIR . 'lib/filters-messages.php';

				$this->msgs = new WpssoPlmFiltersMessages( $plugin, $addon );

				$this->p->util->add_plugin_filters( $this, array( 
					'form_cache_place_names' => 1,
				), $prio = 1000 );	// Run after WPSSO Core's own Standard / Premium filters.
			}
		}

		public function filter_og_type( $og_type, $mod, $is_custom ) {

			if ( $is_custom ) {

				if ( $this->p->debug->enabled ) {

					$this->p->debug->log( 'exiting early: custom og type is true' );
				}

				return $og_type;
			}

			if ( 'place' === $og_type ) {	// Just in case.

				if ( $this->p->debug->enabled ) {

					$this->p->debug->log( 'exiting early: og type is already "place"' );
				}

				return $og_type;
			}

			$place_opts = WpssoPlmPlace::has_place( $mod );	// Returns false or place array.

			if ( empty( $place_opts ) ) {

				if ( $this->p->debug->enabled ) {

					$this->p->debug->log( 'exiting early: no place options found' );
				}

				return $og_type;	// Stop here.
			}

			if ( $this->p->debug->enabled ) {

				$this->p->debug->log( 'returning og_type "place"' );
			}

			return 'place';
		}

		public function filter_og_seed( array $mt_og, array $mod ) {

			if ( $this->p->debug->enabled ) {

				$this->p->debug->mark();
			}

			$place_opts = WpssoPlmPlace::has_place( $mod );	// Returns false or place array.

			if ( empty( $place_opts ) ) {

				if ( $mod[ 'is_home' ] ) {	// Home page (static or blog archive).

					if ( $this->p->debug->enabled ) {

						$this->p->debug->log( 'checking for a home page site organization place ID' );
					}

					if ( $mod[ 'obj' ] ) {	// Just in case.

						$og_type = $mod[ 'obj' ]->get_options( $mod[ 'id' ], 'og_type' );

						if ( 'place' === $og_type ) {

							$org_opts = WpssoSchema::get_site_organization( $mod );

							if ( isset( $org_opts[ 'org_place_id' ] ) && is_numeric( $org_opts[ 'org_place_id' ] ) ) {	// Value is "0" or more.

								$place_opts = WpssoPlmPlace::get_id( $org_opts[ 'org_place_id' ], $mod );
							}
						}
					}
				}
			}

			if ( empty( $place_opts ) ) {

				if ( $this->p->debug->enabled ) {

					$this->p->debug->log( 'exiting early: no place options found' );
				}

				return $mt_og;     // Stop here.
			}

			if ( $this->p->debug->enabled ) {

				$this->p->debug->log( 'returning meta tags for "place"' );
			}

			/**
			 * og:type
			 */
			$mt_og[ 'og:type' ] = 'place';	// Pre-define to optimize.

			/**
			 * place:name
			 * place:telephone
			 * place:street_address
			 * place:po_box_number
			 * place:locality
			 * place:region
			 * place:postal_code
			 * place:country_name
			 */
			foreach ( WpssoPlmPlace::$place_mt as $key => $mt_name ) {

				$mt_og[ $mt_name ] = isset( $place_opts[ $key ] ) && 'none' !== $place_opts[ $key ] ? $place_opts[ $key ] : '';
			}

			/**
			 * place:location:latitude
			 * place:location:longitude
			 * place:location:altitude
			 * og:latitude
			 * og:longitude
			 * og:altitude
			 */
			if ( isset( $place_opts[ 'plm_place_latitude' ] ) && '' !== $place_opts[ 'plm_place_latitude' ] && 
				isset( $place_opts[ 'plm_place_longitude' ] ) && '' !== $place_opts[ 'plm_place_longitude' ] ) {

				foreach( array( 'place:location', 'og' ) as $mt_pre ) {

					$mt_og[ $mt_pre . ':latitude' ]  = $place_opts[ 'plm_place_latitude' ];
					$mt_og[ $mt_pre . ':longitude' ] = $place_opts[ 'plm_place_longitude' ];

					if ( isset( $place_opts[ 'plm_altitude' ] ) && '' !== $place_opts[ 'plm_altitude' ] ) {

						$mt_og[ $mt_pre . ':altitude' ] = $place_opts[ 'plm_place_altitude' ];
					}
				}
			}

			/**
			 * Non-standard meta tags for internal use.
			 */
			$weekdays =& $this->p->cf[ 'form' ][ 'weekdays' ];

			$place_defs = WpssoPlmConfig::$cf[ 'opt' ][ 'def_plm_place' ];

			foreach ( $weekdays as $day_name => $day_label ) {

				foreach ( array(
					'plm_place_day_' . $day_name . '_open'  => 'place:opening_hours:day:' . $day_name . ':open',
					'plm_place_day_' . $day_name . '_close' => 'place:opening_hours:day:' . $day_name . ':close',
				) as $opt_key => $mt_name ) {

					if ( ! empty( $place_opts[ $opt_key ] ) && 'none' !== $place_opts[ $opt_key ] ) {

						$mt_og[ $mt_name ] = $place_opts[ $opt_key ];
					}
				}
			}

			foreach ( array(
				'plm_place_id'                  => 'place:opening_hours:id',
				'plm_place_timezone'            => 'place:opening_hours:timezone',
				'plm_place_midday_close'        => 'place:opening_hours:midday:close',
				'plm_place_midday_open'         => 'place:opening_hours:midday:open',
				'plm_place_season_from_date'    => 'place:opening_hours:season:from_date',
				'plm_place_season_to_date'      => 'place:opening_hours:season:to_date',
				'plm_place_service_radius'      => 'place:business:service_radius',
				'plm_place_currencies_accepted' => 'place:business:currencies_accepted',
				'plm_place_payment_accepted'    => 'place:business:payment_accepted',
				'plm_place_price_range'         => 'place:business:price_range',
				'plm_place_accept_res'          => 'place:business:accepts_reservations',
				'plm_place_menu_url'            => 'place:business:menu_url',
				'plm_place_order_urls'          => 'place:business:order_url',
			) as $opt_key => $mt_name ) {

				if ( isset( $place_opts[ $opt_key ] ) ) {

					if ( 'plm_place_accept_res' === $opt_key ) {

						$mt_og[ $mt_name ] = empty( $place_opts[ $opt_key ] ) ? false : true;

					} elseif ( 'plm_place_order_urls' === $opt_key ) {

						$mt_og[ $mt_name ] = SucomUtil::explode_csv( $place_opts[ $opt_key ] );

					} else {

						$mt_og[ $mt_name ] = $place_opts[ $opt_key ];
					}

				} else {

					$mt_og[ $mt_name ] = '';
				}
			}

			return $mt_og;
		}

		public function filter_json_array_schema_type_ids( $type_ids, $mod ) {

			$place_opts = WpssoPlmPlace::has_place( $mod );	// Returns false or place array.

			if ( empty( $place_opts ) ) {

				if ( $this->p->debug->enabled ) {

					$this->p->debug->log( 'exiting early: no place options found' );
				}

				return $type_ids;	// Stop here.
			}

			/**
			 * Example $place_ids array:
			 *
			 * Array (
			 *	[website]        => 1
			 *	[organization]   => 1
			 *	[person]         => 1
			 *	[local.business] => 1
			 * )
			 */
			$def_schema_type = WpssoPlmConfig::$cf[ 'opt' ][ 'def_plm_place' ][ 'plm_place_schema_type' ];

			$type_id = empty( $place_opts[ 'plm_place_schema_type' ] ) ? $def_schema_type : $place_opts[ 'plm_place_schema_type' ];

			if ( $this->p->debug->enabled ) {

				$this->p->debug->log( 'adding schema type id "' . $type_id . '"' );
			}

			$type_ids[ $type_id ] = true;

			return $type_ids;
		}

		public function filter_json_prop_https_schema_org_potentialaction( $action_data, $mod, $mt_og, $page_type_id, $is_main ) {

			if ( ! $is_main ) {

				return $action_data;
			}

			if ( empty( $mt_og[ 'place:business:order_url' ] ) ) {

				return $action_data;
			}

			$action_data[] = array(
				'@context' => 'https://schema.org',
				'@type'    => 'OrderAction',
				'target'   => $mt_og[ 'place:business:order_url' ],
			);

			return $action_data;
		}

		/**
		 * Add our place names to the place cache array.
		 */
		public function filter_form_cache_place_names( $mixed ) {

			$ret = WpssoPlmPlace::get_names();

			if ( is_array( $mixed ) ) {

				$ret = $mixed + $ret;
			}

			return $ret;
		}
	}
}
