<?php
/**
 * License: GPLv3
 * License URI: https://www.gnu.org/licenses/gpl.txt
 * Copyright 2014-2021 Jean-Sebastien Morisset (https://wpsso.com/)
 */

if ( ! defined( 'ABSPATH' ) ) {

	die( 'These aren\'t the droids you\'re looking for.' );
}

if ( ! class_exists( 'WpssoPlmPlace' ) ) {

	class WpssoPlmPlace {

		public static $place_mt = array(
			'plm_place_name'           => 'place:name',
			'plm_place_name_alt'       => 'place:name_alt',
			'plm_place_desc'           => 'place:description',
			'plm_place_street_address' => 'place:street_address',
			'plm_place_po_box_number'  => 'place:po_box_number',
			'plm_place_city'           => 'place:locality',
			'plm_place_region'         => 'place:region',
			'plm_place_postal_code'    => 'place:postal_code',
			'plm_place_country'        => 'place:country_name',
			'plm_place_phone'          => 'place:telephone',
		);

		public function __construct() {}

		public static function get_ids( $schema_type = '' ) {

			$place_names = self::get_names( $schema_type );

			return array_keys( $place_names );
		}

		/**
		 * Return an associative array of place IDs and names.
		 */
		public static function get_names( $schema_type = '', $add_new = false ) {

			$wpsso =& Wpsso::get_instance();

			if ( $wpsso->debug->enabled ) {

				$wpsso->debug->mark();
			}

			$place_names = array();

			if ( $wpsso->debug->enabled ) {

				$wpsso->debug->log( 'getting multi keys for plm_place_name' );
			}

			$place_names = SucomUtil::get_multi_key_locale( 'plm_place_name', $wpsso->options, $add_none = false );

			if ( ! empty( $schema_type ) && is_string( $schema_type) ) {

				if ( $wpsso->debug->enabled ) {

					$wpsso->debug->log( 'removing places not in schema type: ' . $schema_type );
				}

				$children = $wpsso->schema->get_schema_type_children( $schema_type );

				if ( ! empty( $children ) ) {	// Just in case.

					foreach ( $place_names as $place_id => $name ) {

						if ( ! empty( $wpsso->options[ 'plm_place_schema_type_' . $place_id ] ) && 
							in_array( $wpsso->options[ 'plm_place_schema_type_' . $place_id ], $children ) ) {

							continue;

						} else {

							unset( $place_names[ $place_id ] );
						}
					}
				}

			} elseif ( $wpsso->debug->enabled ) {

				$wpsso->debug->log( 'place schema type not provided - keeping all places' );
			}

			/**
			 * Add 'new' as the last place ID.
			 */
			if ( $add_new ) {

				$next_num = SucomUtil::get_next_num( $place_names );

				$place_names[ $next_num ] = $wpsso->cf[ 'form' ][ 'place_select' ][ 'new' ];
			}

			return $place_names;
		}

		/**
		 * Get a specific place id.
		 * 
		 * Returns an array of localized values.
		 *
		 * $place_id = 'custom' | place ID.
		 * $mixed    = 'default' | 'current' | post ID | $mod array.
		 *
		 * If $place_id is 'custom' then $mixed must be a $mod array.
		 */
		public static function get_id( $place_id, $mixed = 'current' ) {

			$wpsso =& Wpsso::get_instance();

			if ( $wpsso->debug->enabled ) {

				$wpsso->debug->log_args( array( 
					'place_id' => $place_id,
					'mixed'    => $mixed,
				) );
			}

			$plm_opts = array();

			if ( '' === $place_id || 'none' === $place_id ) {	// Just in case.

				return false;

			} elseif ( 'custom' === $place_id ) {	// Read place options from the post, term, or user object.

				if ( ! isset( $mixed[ 'obj' ] ) || ! is_object( $mixed[ 'obj' ] ) ) {	// Just in case.

					if ( $wpsso->debug->enabled ) {

						$wpsso->debug->log( 'exiting early: no module object defined' );
					}

					return false;
				}

				$plm_opts = self::get_md_plm_options( $mixed );	// Always returns and array.

				if ( empty( $plm_opts ) ) {

					$plm_opts = false;

				} else {

					$plm_opts = array_merge( WpssoPlmConfig::$cf[ 'opt' ][ 'def_plm_place' ], $plm_opts );	// Complete the array.

					$plm_opts[ 'plm_place_id' ] = $place_id;	// Make sure the 'plm_place_id' array element exists.
				}

				return $plm_opts;

			} elseif ( is_numeric( $place_id ) ) {

				static $local_cache = array();	// Cache for single page load.

				$locale = SucomUtil::get_locale( $mixed );

				if ( isset( $local_cache[ $place_id ][ $locale ] ) ) {

					if ( $wpsso->debug->enabled ) {

						$wpsso->debug->log( 'returning options from static cache array for place ID ' . $place_id );
					}

					return $local_cache[ $place_id ][ $locale ];
				}

				/**
				 * Get the list of non-localized option names.
				 */
				foreach ( SucomUtil::preg_grep_keys( '/^(plm_place_.*_)' . $place_id . '(#.*)?$/',
					$wpsso->options, $invert = false, $replace = '$1' ) as $opt_pre => $value ) {

					$opt_key = rtrim( $opt_pre, '_' );

					$plm_opts[ $opt_key ] = SucomUtil::get_key_value( $opt_pre . $place_id, $wpsso->options, $mixed );	// Localized value.
				}

				if ( empty( $plm_opts ) ) {

					$plm_opts = false;

				} else {

					$plm_opts = array_merge( WpssoPlmConfig::$cf[ 'opt' ][ 'def_plm_place' ], $plm_opts );	// Complete the array.

					$plm_opts[ 'plm_place_id' ] = $place_id;	// Make sure the 'plm_place_id' array element exists.
				}

				/**
				 * If not a local business or food establishment, remove local business or food establishment options.
				 */
				if ( ! empty( $plm_opts[ 'plm_place_schema_type' ] ) ) {	// Just in case.

					if ( $wpsso->schema->is_schema_type_child( $plm_opts[ 'plm_place_schema_type' ], 'local.business' ) ) {

						if ( ! $wpsso->schema->is_schema_type_child( $plm_opts[ 'plm_place_schema_type' ], 'food.establishment' ) ) {

							unset(
								$plm_opts[ 'plm_place_accept_res' ],		// Accepts Reservations.
								$plm_opts[ 'plm_place_cuisine' ],		// Serves Cuisine.
								$plm_opts[ 'plm_place_menu_url' ],		// Food Menu URL.
								$plm_opts[ 'plm_place_order_urls' ]		// Order Action URL(s).
							);
						}

					} else {

						unset(
							$plm_opts[ 'plm_place_service_radius' ],	// Service Radius.
							$plm_opts[ 'plm_place_currencies_accepted' ],	// Currencies Accepted.
							$plm_opts[ 'plm_place_payment_accepted' ],	// Payment Accepted.
							$plm_opts[ 'plm_place_price_range' ],		// Price Range.
							$plm_opts[ 'plm_place_accept_res' ],		// Accepts Reservations.
							$plm_opts[ 'plm_place_cuisine' ],		// Serves Cuisine.
							$plm_opts[ 'plm_place_menu_url' ],		// Food Menu URL.
							$plm_opts[ 'plm_place_order_urls' ]		// Order Action URL(s).
						);
					}
				}


				if ( $wpsso->debug->enabled ) {

					$wpsso->debug->log( 'saving options to static cache array for place ID ' . $place_id );
				}

				return $local_cache[ $place_id ][ $locale ] = $plm_opts;
			}

			return false;
		}

		/**
		 * Return a text a value for the https://schema.org/address property.
		 */
		public static function get_address( array $plm_opts ) {

			$wpsso =& Wpsso::get_instance();

			if ( $wpsso->debug->enabled ) {

				$wpsso->debug->mark();
			}

			$address = '';

			foreach ( array( 
				'plm_place_street_address',
				'plm_place_po_box_number',
				'plm_place_city',
				'plm_place_region',
				'plm_place_postal_code',
				'plm_place_country',
			) as $opt_key ) {

				if ( isset( $plm_opts[ $opt_key ] ) && $plm_opts[ $opt_key ] !== '' && $plm_opts[ $opt_key ] !== 'none' ) {

					switch ( $opt_key ) {

						case 'plm_place_name':

							$plm_opts[ $opt_key ] = preg_replace( '/\s*,\s*/', ' ', $plm_opts[ $opt_key ] );	// Just in case.

							break;

						case 'plm_place_po_box_number':

							$address = rtrim( $address, ', ' ) . ' #';	// Continue street address.

							break;
					}

					$address .= $plm_opts[ $opt_key ] . ', ';
				}
			}

			return rtrim( $address, ', ' );
		}

		public static function has_place( array $mod ) {

			$wpsso =& Wpsso::get_instance();

			if ( $wpsso->debug->enabled ) {

				$wpsso->debug->mark();
			}

			return self::has_md_place( $mod );	// Returns false or place array.
		}

		public static function has_md_place( array $mod ) {

			$wpsso =& Wpsso::get_instance();

			if ( $wpsso->debug->enabled ) {

				$wpsso->debug->mark();
			}

			if ( ! isset( $mod[ 'obj' ] ) || ! is_object( $mod[ 'obj' ] ) ) {	// Just in case.

				if ( $wpsso->debug->enabled ) {

					$wpsso->debug->log( 'exiting early: no module object defined' );
				}

				return false;
			}

			$plm_opts = self::get_md_plm_options( $mod );	// Always returns an array.

			$place_id = isset( $plm_opts[ 'plm_place_id' ] ) ? $plm_opts[ 'plm_place_id' ] : 'none';	// Just in case.

			if ( 'custom' == $place_id || is_numeric( $place_id ) ) {

				return $plm_opts;
			}

			return false;
		}

		/**
		 * Always returns an array.
		 */
		private static function get_md_plm_options( array $mod ) {

			$wpsso =& Wpsso::get_instance();

			if ( $wpsso->debug->enabled ) {

				$wpsso->debug->mark();
			}

			if ( ! isset( $mod[ 'obj' ] ) || ! is_object( $mod[ 'obj' ] ) ) {	// Just in case.

				if ( $wpsso->debug->enabled ) {

					$wpsso->debug->log( 'exiting early: no module object defined' );
				}

				return array();
			}

			static $local_cache = array();	// Cache for single page load.

			if ( isset( $local_cache[ $mod[ 'name' ] ][ $mod[ 'id' ] ] ) ) {

				if ( $wpsso->debug->enabled ) {

					$wpsso->debug->log( 'returning options from static cache array for ' . $mod[ 'name' ] . ' ID ' . $mod[ 'id' ] );
				}

				return $local_cache[ $mod[ 'name' ] ][ $mod[ 'id' ] ];
			}

			$plm_opts = array();

			$md_opts = $mod[ 'obj' ]->get_options( $mod[ 'id' ] );	// Returns empty string if no meta found.

			if ( isset( $md_opts[ 'schema_place_id' ] ) ) {	// Just in case.

				if ( 'custom' === $md_opts[ 'schema_place_id' ] ) {

					$plm_opts = SucomUtil::preg_grep_keys( '/^plm_/', $md_opts );	// Only return plm options.

				} elseif ( is_numeric( $md_opts[ 'schema_place_id' ] ) ) {	// Allow for 0.

					$plm_opts = self::get_id( $md_opts[ 'schema_place_id' ], $mod );	// Can return false;
				}

				if ( empty( $plm_opts ) ) {

					if ( $wpsso->debug->enabled ) {

						$wpsso->debug->log( 'no plm option keys found' );
					}

					$plm_opts = array();	// Just in case.

				} else {

					if ( $wpsso->debug->enabled ) {

						$wpsso->debug->log( count( $plm_opts ) . ' plm option keys found' );
					}

					$plm_opts[ 'plm_place_id' ] = $md_opts[ 'schema_place_id' ];	// Just in case.
				}
			}

			if ( $wpsso->debug->enabled ) {

				$wpsso->debug->log( 'result saved to static cache array for ' . $mod[ 'name' ] . ' ID ' . $mod[ 'id' ] );
			}

			return $local_cache[ $mod[ 'name' ] ][ $mod[ 'id' ] ] = $plm_opts;
		}
	}
}
